/// WordToOPML
/// 
/// Purpose: Convert word documents to OPML format, using the Hyperscope OPML extensions
///    documented at http://www.hyperscope.org
///
/// Description: Extracts TEXT from Microsoft Word, equivalent to viewing all levels in Word's Outline
//     mode.
/// 
/// Requires: Microsoft Word v 11.0 (or greater) to be installed on the machine
///
/// Author: Adam Cheyer (adam.cheyer@sri.com)
/// Date: 09/07/06
/// 
/// License: LGPL
/// 

// Change Log
//  1.0.0.0    Initial version.  Creates OPML from Word Outline text
//  1.0.1.0    Fixed problem with "owerName": now reading correctly from document "Author" property
//             Remove blank lines from generated outline

// Imports
using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using System.IO;

namespace wordToOPML
{
	/// <summary>
	/// frmMain is the primary GUI for wordToOPML application
	/// </summary>
	public class frmMain : System.Windows.Forms.Form
	{

        /// <summary>
        /// Class variables
        /// </summary>
        private int parNum = 1;
        XmlTextWriter outfil;

        /// <summary>
		/// User Interface Objects
		/// </summary>
		private System.Windows.Forms.MenuItem mnuFile;
		private System.Windows.Forms.MenuItem mnuExit;
        private System.Windows.Forms.MainMenu mainMenu;

        private IContainer components;
        private System.Windows.Forms.OpenFileDialog openFileDialog;
        private ProgressBar progressBar1;
        private MenuItem mnuAbout;
        private Button btnLoadSource;

		public frmMain()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmMain));
            this.mainMenu = new System.Windows.Forms.MainMenu(this.components);
            this.mnuFile = new System.Windows.Forms.MenuItem();
            this.mnuExit = new System.Windows.Forms.MenuItem();
            this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
            this.progressBar1 = new System.Windows.Forms.ProgressBar();
            this.btnLoadSource = new System.Windows.Forms.Button();
            this.mnuAbout = new System.Windows.Forms.MenuItem();
            this.SuspendLayout();
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuFile,
            this.mnuAbout});
            // 
            // mnuFile
            // 
            this.mnuFile.Index = 0;
            this.mnuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuExit});
            this.mnuFile.Text = "&File";
            // 
            // mnuExit
            // 
            this.mnuExit.Index = 0;
            this.mnuExit.Text = "E&xit";
            this.mnuExit.Click += new System.EventHandler(this.mnuExit_Click);
            // 
            // openFileDialog
            // 
            this.openFileDialog.Multiselect = true;
            this.openFileDialog.Title = "SAMS Parser.Net";
            // 
            // progressBar1
            // 
            this.progressBar1.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.progressBar1.Location = new System.Drawing.Point(0, 70);
            this.progressBar1.Name = "progressBar1";
            this.progressBar1.Size = new System.Drawing.Size(236, 28);
            this.progressBar1.TabIndex = 1;
            // 
            // btnLoadSource
            // 
            this.btnLoadSource.Dock = System.Windows.Forms.DockStyle.Fill;
            this.btnLoadSource.Location = new System.Drawing.Point(0, 0);
            this.btnLoadSource.Name = "btnLoadSource";
            this.btnLoadSource.Size = new System.Drawing.Size(236, 70);
            this.btnLoadSource.TabIndex = 2;
            this.btnLoadSource.Text = "Load Word File";
            this.btnLoadSource.UseVisualStyleBackColor = true;
            this.btnLoadSource.Click += new System.EventHandler(this.btnLoadSource_Click);
            // 
            // mnuAbout
            // 
            this.mnuAbout.Index = 1;
            this.mnuAbout.Text = "&About";
            this.mnuAbout.Click += new System.EventHandler(this.mnuAbout_Click);
            // 
            // frmMain
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(236, 98);
            this.Controls.Add(this.btnLoadSource);
            this.Controls.Add(this.progressBar1);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Menu = this.mainMenu;
            this.Name = "frmMain";
            this.Text = "Word To OPML";
            this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}

        /// <summary>
        /// Returns false for illegal characters
        /// </summary>
        /// <param name="c"></param>
        /// <returns></returns>
        private bool OKChar(char c)
        {
            bool ok = ((c >= ' ') && (c <= 'z'));
            string bad = "<>&";
            if (bad.IndexOf(c) >= 0)
                ok = false;
            return ok;
        }

        // Cleans string to only legal characters
        private string CleanString(string s)
        {
            string cs = "";
            for (int i = 0; i < s.Length; i++)
                if (OKChar(s[i]))
                    cs = cs + s[i];
            return cs;
        }


        /// <summary>
        /// Recursive function to nest outline elements by paragraph OutlineLevel
        /// </summary>
        /// <param name="pars">A list of paragraphs in the current document</param>
        private void writeSection(Word.Paragraphs pars)
        {
            bool done = false;
            bool writeClose = false;
            string t = CleanString(pars[parNum].Range.Text.Trim());

            if (!t.Equals(""))
            {
                outfil.WriteStartElement("outline");
                outfil.WriteAttributeString("text", CleanString(pars[parNum].Range.Text.Trim()));
                outfil.WriteAttributeString("hs:nid", "0" + parNum.ToString());
                writeClose = true;
            }

            Word.WdOutlineLevel currentLevel = pars[parNum].OutlineLevel;

            parNum = parNum + 1;

            progressBar1.Value = parNum;
            Application.DoEvents();

            while ((parNum < pars.Count) && !done)
            {
                if (pars[parNum].OutlineLevel == currentLevel)
                {
                   if (writeClose)
                   {
                       outfil.WriteEndElement();
                       writeClose = false;
                   }
                   t = CleanString(pars[parNum].Range.Text.Trim());

                   if (!t.Equals(""))
                   {
                       outfil.WriteStartElement("outline");
                       outfil.WriteAttributeString("text", t);
                       outfil.WriteAttributeString("hs:nid", "0" + parNum.ToString());
                       writeClose = true;
                   }
                   parNum = parNum + 1;
                   progressBar1.Value = parNum;
                   Application.DoEvents();
               }
                else if (pars[parNum].OutlineLevel > currentLevel)
                {
                    writeSection(pars);
                }
                else
                {
                    done = true;
                }
            }
            if (writeClose)
            {
                outfil.WriteEndElement();
                writeClose = false;
            }
        }


        /// <summary>
        /// Lookup the property for a document.  
        /// Properties may be: 
        ///   "Title", "Subject", "Author", "Keywords", "Revision Number", 
        ///   "Creation Date", "Last Save Time"
        /// </summary>
        /// <param name="documentProperties">List of document properties</param>
        /// <param name="propName">Name of property to return</param>
        /// <returns>Property value</returns>
        private string GetDocumentProperty(object documentProperties, string propName)
        {
            string propValue = "";
            try
            {
                Type propertyType = documentProperties.GetType();

                object property = propertyType.InvokeMember("Item",
                        System.Reflection.BindingFlags.Default |
                        System.Reflection.BindingFlags.GetProperty,
                        null,
                        documentProperties,
                        new object[] { propName });

                Type validatedType = property.GetType();

                propValue = validatedType.InvokeMember("Value",
                    System.Reflection.BindingFlags.Default |
                    System.Reflection.BindingFlags.GetProperty,
                    null,
                    property,
                    new object[] { }).ToString();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception: {0}", ex.ToString());
            }

            return propValue;
        }

		/// <summary>
		/// Get source document. Open a FileDialog window for user to select single/multiple files for
		/// parsing.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
        private void btnLoadSource_Click(object sender, System.EventArgs e)
		{
			if( openFileDialog.ShowDialog() == DialogResult.OK )
			{

				object	fileName = openFileDialog.FileName;
                String  outFileName = (string) fileName;

                outFileName = outFileName.ToLower().Replace(".doc",".opml");

                DateTime modDate = File.GetLastWriteTime(fileName.ToString());
                DateTime createDate = File.GetCreationTime(fileName.ToString());

                // Date standard used in RFC822
                string dateFormat = "ddd dd MMM yyyy HH:mm:ss";
								
				object	readOnly	= false;
				object	visible		= true;				
  			    object	missing		= System.Reflection.Missing.Value;
                object  saveChanges = false;

                btnLoadSource.Text = "Loading document in Word...";
                btnLoadSource.Enabled = false;

                Application.DoEvents();

         		Word.ApplicationClass appWord = new Word.ApplicationClass();

				appWord.Visible = false;
				// vk_word_app.Activate();

				// Open the document
				Word.Document docWord = appWord.Documents.Open( ref fileName,
					ref missing, ref readOnly, ref missing, ref missing,
               ref missing, ref missing, ref missing, ref missing,
               ref missing, ref missing, ref visible, ref missing, ref missing, 
               ref missing, ref missing );

                btnLoadSource.Text = "Converting document...";
                Application.DoEvents();

                // get document properties for use later
                object documentProperties = docWord.BuiltInDocumentProperties;

                // Start writing the file
                outfil = new XmlTextWriter(outFileName, System.Text.Encoding.UTF8);

                progressBar1.Minimum = 1;
                progressBar1.Value = 1;
                progressBar1.Maximum = docWord.Paragraphs.Count;

                try
                {
                   outfil.Formatting = Formatting.Indented;
                   outfil.Indentation= 3;
                   outfil.Namespaces = false;

                   outfil.WriteStartDocument();

                   String instruction = "type=\"text/xsl\" href=\"/hyperscope/src/client/lib/hs/xslt/hyperscope.xsl\"";
                   outfil.WriteProcessingInstruction("xml-stylesheet", instruction);
       
                   outfil.WriteStartElement("opml");
                     outfil.WriteAttributeString("xmlns:hs", "http://hyperscope.org/hyperscope/opml/public/2006/05/09");
                     outfil.WriteAttributeString("hs:version", "1.0");
                     outfil.WriteAttributeString("version", "2.0");

                      outfil.WriteStartElement("head");
                        outfil.WriteAttributeString("hs:left-label-delim", "");
                        outfil.WriteAttributeString("hs:right-label-delim", "");
                        outfil.WriteAttributeString("hs:nidcount", docWord.Paragraphs.Count.ToString());
                        outfil.WriteElementString("ownerName", GetDocumentProperty(documentProperties, "Author"));
                        outfil.WriteElementString("dateCreated", createDate.ToString(dateFormat) + " GMT");
                        outfil.WriteElementString("dateModified", modDate.ToString(dateFormat) + " GMT");
                      outfil.WriteEndElement();   // Head

                      outfil.WriteStartElement("body");
                    
                   // Paragraph collection is 1-based
                   parNum = 1;
                   while (parNum <= docWord.Paragraphs.Count)
                   {
                      writeSection(docWord.Paragraphs);
                   }

                   outfil.WriteEndElement();  // Body
                   outfil.WriteEndElement();  // OPML
                   outfil.WriteEndDocument();

                   outfil.Flush();
                }
                catch(Exception ex)
                {
                    Console.WriteLine("Exception: {0}", ex.ToString());
                }
                finally
                {
                    if (outfil != null)
                    {
                        outfil.Close();
                        progressBar1.Value = 1;
                        btnLoadSource.Enabled = true;
                        btnLoadSource.Text = "Load Word File";
                        Application.DoEvents();
                    }
                }

				// close word application
				appWord.Quit( ref saveChanges, ref missing, ref missing );
			}
		}

        /// <summary>
        /// Exit the program
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
		private void mnuExit_Click(object sender, System.EventArgs e)
		{
			Close();	
		}

        private void mnuAbout_Click(object sender, EventArgs e)
        {
            frmAboutBox frmAbout = new frmAboutBox();
            frmAbout.ShowDialog();
        }

	}
}
